package visl.rasmus.handler;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.ImageObserver;
import java.util.Enumeration;
import java.util.Vector;

public class Sprite extends MultiComponent implements ImageObserver {
    protected Vector images;
    protected int index, w, h;
    protected boolean animate;

    public Sprite(Sprite sp) {
	index = 0;
	w = sp.w;
	h = sp.h;
	images = new Vector();
	for (Enumeration en = sp.images.elements(); en.hasMoreElements();) {
	    images.addElement(en.nextElement());
	}
	animate = sp.animate;
    }
    public Sprite() {
	images = new Vector();
	animate = false;
	index = 0;
    }
    public synchronized void addImage(Image img) { 
	images.addElement(img); 
	if (img != null) {
	    w = img.getWidth(this);
	    h = img.getHeight(this);
	}
    }
    public synchronized void clear() {
	images.removeAllElements();
	index = 0;
    }
    public synchronized void paint(Graphics g) {
	if (images.size() > 0) {
	    Image im = (Image)images.elementAt(index);
	    if (im != null) {
		if (getParent() == null)
		    g.drawImage(im, x - w/2, y - h/2, this);
		else
		    g.drawImage(im, x, y, this);
	    }
	}
    }
    public synchronized boolean isAnimate() { return animate; }
    public synchronized void setAnimate(boolean an) { animate = an; }
    public synchronized int nextImage() {
	if (index < images.size()-1) index++;
	else index = 0;
	return index;
    }
    public synchronized void setImage(int ix) { index = ix; }
    public void updateGraphics(Graphics g, Object o) {
	paint(g);
    }
    public boolean imageUpdate(Image img, int infoflags, int x, int y,
			       int width, int height) {
	return false;
    }
    public Image getImage(int i) {
	if (i >= 0 && i < images.size()) return (Image)images.elementAt(i);
	return null;
    }
    public Image getImage() { 
	if (images.size() == 0 || index >= images.size() ||
	    index < 0) return null;
	return (Image)images.elementAt(index); 
    }
    public Dimension size() {
	return new Dimension(w,h);
    }
    public int getImageCount() { return images.size(); }
    public Dimension preferredSize() {
	return new Dimension(w,h);
    }
}

