package visl.rasmus.handler;

import java.awt.Color;
import java.awt.FontMetrics;

/** Container class for TextScrollHandler 
 * @author Rasmus Lock Larsen, VISL Februar 2001.
 * @see TextScrollHandler */
public class TextScrollComponent implements Cloneable {
	static FontMetrics fm = null;
	static int sH = 0, sM = 0;
	public String text;
	public String form;
	public double ax, ay; // Accelerations
	public double vx, vy; // Speeds
	public double x, y; // Positions
	public int width, height, awidth;
	public Color color;
	public boolean hasbeenhit;
	public TextScrollComponent(TextScrollComponent tsc) {
		this.text = tsc.text;
		this.form = tsc.form;
		this.ax = tsc.ax;
		this.ay = tsc.ay;
		this.vx = tsc.vx;
		this.vy = tsc.vy;
		this.x = tsc.x;
		this.y = tsc.y;
		this.width = tsc.width;
		this.height = tsc.height;
		this.color = tsc.color;
		this.hasbeenhit = tsc.hasbeenhit;
	}
	public TextScrollComponent(String text, String form) {
		this(text, 0, 0, 0, form);
	}
	public TextScrollComponent(
		String text,
		double speed,
		int x,
		int y,
		String form) {
		this.text = text;
		this.form = form;
		this.ax = 0;
		this.ay = 0;
		this.vx = speed;
		this.vy = 0;
		this.x = x;
		this.y = y;
		if (fm != null) {
			awidth = fm.stringWidth(text);
			height = fm.getHeight();
			
			//ensures a minimum size for short words
			if (awidth < sM && text.length() > 0)
				width = sM;
			else
				width = awidth;
			if (height < sH)
				height = sH;
		} else {
			width = 0;
			height = 0;
		}
		color = Color.white;
		hasbeenhit = false;
	}
	public String toString() {
		return "TextScrollComponent["
			+ text
			+ ","
			+ x
			+ ","
			+ y
			+ ","
			+ width
			+ ","
			+ height
			+ ","
			+ vx
			+ "]";
	}
	/** Returns <CODE>true</CODE> if (hx, hy) is within the component */
	public boolean isHit(int hx, int hy) {
		return (
			!hasbeenhit
				&& (hx >= x)
				&& (hx <= (x + width))
				&& (hy <= y)
				&& (hy >= (y - height)));
	}
	static public void setFontMetrics(FontMetrics fms) {
		fm = fms;
		sM = fm.stringWidth("MM");
		sH = fm.getHeight();
	}
	static public FontMetrics getFontMetrics() {
		return fm;
	}
}
