package visl.rasmus.handler;

import java.awt.Color;
import java.awt.Graphics;
import java.util.Enumeration;
import java.util.Vector;

import visl.rasmus.utils.TimerListener;

/** This class implements a handler to take care of numereous scrolling texts
 * @author Rasmus Lock Larsen, VISL Februar 2001. */
public class TextScrollHandler implements GraphicsHandler, TimerListener {
	protected static boolean colorchange = false;

	protected Vector components;
	protected long oldtime, newtime;
	protected int rangex, rangey;
	protected int misses;
	protected String form;

	public TextScrollHandler() {
		components = new Vector();
		oldtime = System.currentTimeMillis();
		rangex = Integer.MAX_VALUE;
		rangey = Integer.MAX_VALUE;
	}
	public synchronized void add(TextScrollComponent tc) {
		//System.out.println("Added: " + tc);
		components.addElement(tc);
		//System.out.println("Add: Number of components: " + components.size());
	}
	public synchronized TextScrollComponent isHit(int x, int y) {
		boolean hitit = false;
		for (Enumeration en = components.elements();
			en.hasMoreElements() && !hitit;
			) {
			TextScrollComponent tsc = (TextScrollComponent) en.nextElement();
			hitit = tsc.isHit(x, y);
			if (hitit) {
				tsc.hasbeenhit = true;
				// Go up into the air
				tsc.vy = -0.5 * Math.abs(tsc.vx);
				tsc.vx = 0.5 * tsc.vx;
				tsc.ay = 0.0001;
				return tsc;
			}
		}
		return null;
	}
	/** Called by the Timer */
	public synchronized void tick() {
		newtime = System.currentTimeMillis();
		long t = newtime - oldtime;
		// Update all the positions
		for (Enumeration en = components.elements(); en.hasMoreElements();) {
			TextScrollComponent tsc = (TextScrollComponent) en.nextElement();

			tsc.vx = tsc.vx + t * tsc.ax;
			tsc.vy = tsc.vy + t * tsc.ay;
			tsc.x = tsc.x + t * tsc.vx;
			tsc.y = tsc.y + t * tsc.vy;
			if (colorchange) {
				Color c = tsc.color;
				float hsbs[] = new float[3];
				Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), hsbs);
				hsbs[1] = 1;
				hsbs[2] =
					(float) (tsc.y > rangey / 2
						? 1.5 - tsc.y / rangey
						: 2 * tsc.y / rangey);
				if (hsbs[2] < 0)
					hsbs[2] = 0;
				hsbs[0] = (float) (0.2 * tsc.x / rangex + 0.8 * tsc.y / rangey);
				if (hsbs[0] > 1)
					hsbs[0] = 0;
				tsc.color = Color.getHSBColor(hsbs[0], hsbs[1], hsbs[2]);
			}
			if ((tsc.x < -tsc.width && tsc.vx <= 0)
				|| (tsc.y > rangey && tsc.vy >= 0)
				|| (tsc.x > rangex + tsc.width && tsc.vx >= 0)
				|| (tsc.y < 0 && tsc.vy <= 0)) {
				if (tsc.form.equals(form) && !tsc.hasbeenhit) {
					misses++;
				}
				components.removeElement(tsc);
				//components.trimToSize();
			}
		}
		oldtime = newtime;
	}
	/** Comes from the GraphicsHandler */
	public synchronized void updateGraphics(Graphics g, Object arg) {
		//g.setFont((Font)Miscellaneous.get("scrollfont"));
		for (Enumeration en = components.elements(); en.hasMoreElements();) {
			TextScrollComponent tsc = (TextScrollComponent) en.nextElement();
			if (tsc.x < rangex && tsc.y < rangey + tsc.height) {
				g.setColor(tsc.color);
				g.drawString(
					tsc.text,
					(int) tsc.x + (tsc.width - tsc.awidth) / 2,
					(int) tsc.y);
			}
		}
	}
	public synchronized boolean hasComponents() {
		return !(components.isEmpty());
	}
	public synchronized int numComponents() {
		return components.size();
	}
	public synchronized void setSize(int x, int y) {
		rangex = x;
		rangey = y;
	}
	public synchronized void clear() {
		components.removeAllElements();
		misses = 0;
	}
	public synchronized void remove(TextScrollComponent tsc) {
		components.removeElement(tsc);
	}
	public synchronized void colorChange(boolean b) {
		colorchange = b;
	}
	public synchronized void setMiss(int m) {
		misses = m;
	}
	public synchronized int getMiss() {
		return misses;
	}
	public synchronized void setForm(String f) {
		form = f;
	}
}
