package visl.rasmus.hitserver.data;

import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * This class extends the Word class in order to add morphology 
 * information to the word.
 * @author brask
 */
public class MorphWord extends Word {
	public ArrayList morph;
	public String baseform;
	/**
	 * The morphological information is stored in an ArrayList.
	 * The first word in the <CODE>morphology</CODE>-string is stored
	 * separetly since I expect it to be the base form of the word.
	 */
	public MorphWord(String text, String form, String morphology) {
		super(text, form);
		// Make sure :,; and \ are de-escaped
		morph = new ArrayList();
		StringTokenizer tok = new StringTokenizer(morphology, ",");
		if (tok.hasMoreTokens())
			baseform = tok.nextToken();
		for (; tok.hasMoreTokens();) {
			morph.add(tok.nextToken());
		}
	}
	public ArrayList getMorph() {
		return morph;
	}
	public String getBaseform() {
		return baseform;
	}
	public String toString() {
		return Sentence.escape(text) + ":" + Sentence.escape(form) + ":" + morph;
	}
}
