package visl.rasmus.hitserver.data;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.Random;
import java.util.Vector;

public class SentenceDB {
    protected Vector items;
    protected Random randgen;

    public SentenceDB() {
	items = new Vector();
	randgen = new Random(System.currentTimeMillis());
    }
    public void readSentences(String filename, PrintStream os, 
			      String encoding) {
	int linenum = 1;
	String s = null;
	String form = null;
	try {
	    URL url = new URL(filename);
	    BufferedReader din = null;
	    if (encoding != null)
		din = new BufferedReader(new InputStreamReader(url.
							       openStream(), 
							       encoding));
	    else
		din = new BufferedReader(new InputStreamReader(url.
							       openStream()));
	    s = din.readLine();
	    Sentence sentence = null;
	    Vector sv = new Vector();
	    boolean inanalysis = false;
	    int ix = -1, jx = -1, kx = -1;
	    while (s != null) {
		// Format: <F>:<f>[(...)]\t <word>
		s = s.trim();
		if (s.equals("A1")) inanalysis = true;
		//else if (s.equals("")) inanalysis = false;
		else if (s.toLowerCase().startsWith("source:") ||
			 (s.length() > 0 && 
			  s.charAt(0) > '0' && s.charAt(0) < '9')) 
		    inanalysis = false;
		
		ix = s.indexOf(':');
		if (ix != -1 && inanalysis) {
		    kx = s.indexOf('\t', ix);
		    if (kx != -1) {
			if (sentence == null) sentence = new Sentence();
			jx = s.indexOf('(', ix);
			String text = s.substring(kx+1);
			form = null;
			if (jx != -1 && jx < kx) {
			    form = s.substring(s.indexOf(':')+1,jx);
			} else form = s.substring(s.indexOf(':')+1, kx);
			form = filterForm(form);
			sentence.addWord(text, form);
		    }
		} else if ((s.startsWith("A1") ||
			    s.startsWith("&&")) && 
			   sentence != null) {
		    // Next analysis
		    if (sv.isEmpty()) sv.addElement(sentence);
		    else {
			boolean isnew = true;
			for (int i = 0; i < sv.size() && isnew; i++) {
			    isnew = !sentence.equals(sv.elementAt(i));
			}
			if (isnew) sv.addElement(sentence);
		    }
		    if (s.startsWith("A1")) {
			for (int i = 0; i < sv.size(); i++)
			    items.addElement(sv.elementAt(i));
			sv.removeAllElements();
		    }
		    sentence = null;
		}
		s = din.readLine();
		linenum++;
	    }
	    if (sentence != null) items.addElement(sentence);
	    din.close();
	} catch (Exception e) {
	    os.println("Exception while reading ["+filename+"] at line " +
		       linenum + " ("+s+")("+form+"):"+e);
	}
    }
    public int size() { return items.size(); }
    public Enumeration getAll() { return items.elements(); }
    public Sentence getSentence(int ix) { 
	if (ix >= 0 && ix < items.size()) return (Sentence)items.elementAt(ix);
	return null;
    }
    public Sentence getRandomSentence() {
	return (Sentence)items.
	    elementAt((int)(randgen.nextDouble()*(items.size()-1)));
    }
    public Sentence getRandomSentence(int min, int max) {
	int sindex = (int)(randgen.nextDouble()*(items.size()-1));
	Sentence s = getSentence(sindex);
	int cnt = 1, l = s.size();
	while (cnt <= items.size() && (l < min || l > max)) {
	    s = getSentence(sindex + cnt);
	    l = s.size();
	    cnt++;
	}
	if (cnt >= items.size()) return null;
	return s;
    }
    public Sentence getRandomSentenceWithForm(String form, int min, int max) {
	int sindex = (int)(randgen.nextDouble()*(items.size()-1));
	Sentence s = getSentence(sindex);
	int l = s.size();
	int cnt = 1;
	int n = items.size();
	if (form.indexOf("|") == -1) form = "|"+form+"|";
	while ((l < min || l > max || !s.containsForm(form)) &&
	       cnt <= items.size()) {
	    s = getSentence((sindex + cnt) % n);
	    l = s.size();
	    cnt++;
	}
	if (cnt >= items.size()) return null;
	return s;
    }
    protected String filterForm(String f) {
	if (f.indexOf('-') != -1) f = f.substring(0,f.indexOf('-'));
	if (f.indexOf('(') != -1) f = f.substring(0,f.indexOf('('));
	if (f.equals("in")) f = "intj";
	else if (f.equals("prop")) f = "n";
	
	return f;
    }
    public void clear() { items.removeAllElements(); }
}
