package visl.rasmus.resources;

import java.applet.Applet;
import java.applet.AudioClip;
import java.io.DataInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;

/**
* ResourceBundle-like class for encapsulation of the comments of a given 
* language.
* @author Rasmus Lock Larsen, VISL, Nov. 2000
*/
public class Comments {
    public static final int 
	POSITIVE = 0,
	NEGATIVE = 1,
	HELP = 2,
	GOOD = 3,
	FAIR = 4,
	POOR = 5,
	GOTHELP = 6,
	WELCOME = 7,
	DOUBLEWORK = 8;
    static Applet applet = null;
    static String soundbase = null;
    static String commentURL = null;
    static String soundURL = null;
    /*
     * The {type, comment} pairs as array.
     */
    static String[][] commentarray;
    /*
     * The {type, sound} pairs as array.
     */
    static String[][] soundarray;
    /*
     * The {type, comment} pairs as Hashtable.
     */
    static Hashtable comments = null;
    /*
     * The {type, sound} pairs as Hashtable.
     */
    static Hashtable sounds = null;
	

    /**
     * Read comments from url.
     *@param commentUrl url to read comments from.
     *@param soundUrl url to read sounds from.
     *@exception MalformedURLException if any of the URL
     */
    public static void localize(String commentUrl, String soundUrl) throws 
    MalformedURLException, IOException{
	commentURL = commentUrl;
	soundURL = soundUrl;
    }
	
	
    /**
     * Read a name=value set from an url
     * Format of the file pointed to: ({num}'='{String}'\n')*
     */
    static String[][] readSet(String url) throws 
    MalformedURLException, IOException{
	return readSet(new URL(url));
    }
	
    /**
     * Read a name=value set from an url
     * Format of the file pointed to: ({num}'='{String}'\n')*
     */
    static String[][] readSet(URL url) throws MalformedURLException, 
    IOException {
	Vector set = new Vector();
	String s = null;
	DataInputStream d = null;
	d = new DataInputStream(url.openStream());
	s = d.readLine(); // read the first line
	s.trim();
	String abbr,full; // abbreviated and full name of symbol
	int equ; // position of the '=' character
	while(s != null){ // while not EOF
	    equ = s.indexOf('=');
	    if (s.startsWith("//")) ; // Comments
	    else if(equ>=0){
		abbr = s.substring(0,equ);
		full = s.substring(equ+1,s.length());
		set.addElement(new String[]{abbr,full}); 
		// Add the (abbr,full name) pair to the set of symbols
	    }
	    s = d.readLine(); // read the next line.
	    if (s != null) s.trim();
	}
		
	int n = set.size(); // number of symbols
	String[][] symbols = new String[n][];
	for(int i=0; i<n; i++){
	    symbols[i] = (String[]) set.elementAt(i);
	}
	return symbols;
    }
	
    /**
     * Initialize tables. Called by the localize method.
     *@param commentset Array of {"type", "full comment"} pairs.
     *@param soundset Array of {"type", "sound"} pairs.
     */
    static void initTables(String[][] commentset, String[][] soundset) {
	if (commentset != null) {
	    commentarray = commentset;
	    comments = new Hashtable();
	    
	    for (int i=0; i < commentarray.length; i++) {
		Integer key = null;
		try {
		    key = Integer.valueOf(commentarray[i][0]);
		} catch (NumberFormatException e) {
		    System.out.println("Error reading comment (" +
				       commentarray[i][1] + ")");
		}
		if (key != null) {
		    if (comments.containsKey(key)) {
			((Vector)comments.get(key)).
			    addElement(commentarray[i][1]);
		    } else {
			Vector v = new Vector();
			v.addElement(commentarray[i][1]);
			comments.put(key, v);
		    }
		}
	    }
	}
	if (soundset != null && applet != null) {
	    soundarray = soundset;
	    sounds = new Hashtable();
	    System.out.print("Loading sounds");
	    // Get all the sounds
	    for (int i=0; i < soundarray.length; i++) {
		Integer key = null;
		try {
		    key = Integer.valueOf(soundarray[i][0]);
		} catch (NumberFormatException e) {
		    System.out.println("Error reading sound (" +
				       soundarray[i][1] + ")");
		}
		try {
		    if (key != null) {
			if (sounds.containsKey(key)) {
			    ((Vector)sounds.get(key)).
				addElement(applet.
					   getAudioClip(new URL(soundbase),
							soundarray[i][1]));
			    System.out.print(".");
			} else {
			    Vector v = new Vector();
			    v.addElement(applet.
					 getAudioClip(new URL(soundbase),
						      soundarray[i][1]));
			    sounds.put(key, v);
			    System.out.print(".");
			}
		    }
		} catch (MalformedURLException e) {
		    System.out.println("Malformed sound-URL: " + soundbase +
				       soundarray[i][1]);
		}
	    }
	    System.out.println("done!");
	    //new SoundLoader(applet, soundbase, sounds, soundarray);
	}
    }	
    
    //
    // Extraction methods for comments
    //
    public static String getComment(int t) {
	Integer type = new Integer(t);
	if (comments == null) loadComments();
	Vector v = (Vector)comments.get(type);
	if (v == null) return "";
	return (String)v.elementAt((int)(Math.random()*v.size()));
    }
    public static AudioClip getSound(int t) {
	Integer type = new Integer(t);
	if (sounds == null) loadSounds();
	Vector v = (Vector)sounds.get(type);
	if (v == null) {
	    System.out.println("No Sounds of type " + type.intValue());
	    return null;
	}
	// The first sound is used in 75% of the cases...
	if (Math.random() >= 0.25) return (AudioClip)v.elementAt(0);
	else return (AudioClip)v.elementAt((int)(Math.random()*v.size()));
    }
    public static void playSound(int t) {
	AudioClip ac = getSound(t);
	if (ac != null) ac.play();
    }
    public static void setApplet(Applet aplt, String sndbase) {
	applet = aplt;
	soundbase = sndbase;
    }
    static void loadSounds() {
	try {
	    soundarray = readSet(soundURL);
	    initTables(null, soundarray);
	} catch (Exception e) {
	    System.out.println("Exception while loading sounds...");
	}
    }
    static void loadComments() {
	try {
	    commentarray = readSet(commentURL);
	    initTables(commentarray, null);
	} catch (Exception e) {
	    System.out.println("Exception while loading comments...");
	}
    }
}



