package visl.wordfall.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

import visl.rasmus.handler.DropElement;
import visl.rasmus.hitserver.data.Word;
import visl.wordfall.resources.Colors;
import visl.wordfall.resources.Miscellaneous;

public class FallWord extends DropElement {
    protected Word word;
    protected int x, y, h;
    protected boolean hasfalled;
    /** Used as a transparent label */
    public FallWord(String w) {
	this(new Word(w, null));
    }
    public FallWord(FallWord fw) {
	this(fw.word);
	fw.hasfalled = true;
	fw.setForeground(Colors.get("fallword.highlight"));
	x = fw.x;
	y = fw.y;
	resize(preferredSize());
    }

    public FallWord(Word word) {
	this.word = word;
	setFont((Font)Miscellaneous.get("dropfont"));
	setForeground(Color.white);
	x = 0;
	y = 0;
	hasfalled = false;
    }
    public Dimension preferredSize() {
	Dimension d = new Dimension();
	FontMetrics fm = getFontMetrics(getFont());
	if (fm != null) {
	    d.width = fm.stringWidth(word.text) + 8;
	    h = fm.getHeight();
	    d.height = h+4;
	}
	return d;
    }
    public void paint(Graphics g) {
	if (isVisible()) {
	    g.setColor(getForeground());
	    if (getParent() == null) {
		FontMetrics fm = getFontMetrics(getFont());
		int w = fm.stringWidth(word.text);
		g.setFont(getFont());
		g.drawString(word.text, x-(w/2), h+y+2);
	    } else {
		g.setFont(getFont());
		Rectangle r = bounds();
		Dimension d = size();
		FontMetrics fm = getFontMetrics(getFont());
		int w = fm.stringWidth(word.text);
		g.drawString(word.text, r.x + (d.width-w)/2, 
			     r.y + (d.height+h)/2);
	    }
	}
    }
    public void setPosition(int x, int y) {
	this.x = x;
	this.y = y;
    }
    public Point getPosition() {
	return new Point(x, y);
    }
    public boolean hasFalled() { return hasfalled; }
    public void setFalled(boolean b) { 
	hasfalled = b; 
	if (hasfalled) {
	    setForeground(Colors.get("fallword.hasfalled"));
	}
    }
    public String getForm() { return word.form; }
    public String getText() { return word.text; }
    public String toString() {
	String res = getClass().getName()+"[";
	res += "word="+word;  
	res += ",x="+x;
	res += ",y="+y;
	res += ",h="+h;
	res += ",hasfalled="+hasfalled;
	res += "]";
	return res;
    }
}


