package visl.wordfall.gui;

import java.awt.BorderLayout;
import java.awt.CheckboxMenuItem;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Label;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.Panel;
import java.awt.Point;
import java.awt.Rectangle;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Random;
import java.util.Vector;

import visl.WordFall;
import visl.rasmus.gui.CheckboxMenuGroup;
import visl.rasmus.gui.HighScoreNameDialog;
import visl.rasmus.gui.ImageCanvas;
import visl.rasmus.gui.ImagePanel;
import visl.rasmus.gui.ProgressOMeter;
import visl.rasmus.gui.YesNoDialog;
import visl.rasmus.handler.DropElement;
import visl.rasmus.handler.DropElementHandler;
import visl.rasmus.handler.TextScrollComponent;
import visl.rasmus.handler.TextScrollHandler;
import visl.rasmus.hitserver.client.HitClientPHP;
import visl.rasmus.hitserver.data.HighScoreItem;
import visl.rasmus.hitserver.data.Sentence;
import visl.rasmus.hitserver.data.Word;
import visl.rasmus.resources.Sounds;
import visl.rasmus.utils.Timer;
import visl.rasmus.utils.TimerListener;
import visl.wordfall.resources.Colors;
import visl.wordfall.resources.Miscellaneous;
// //import visl.resources.DebugFrame;
import visl.resources.Dict;
import visl.resources.ShortToLong;

public class MainFrame extends Frame implements TimerListener {
	public static final int WIDTH = 300,
		HEIGHT = 450,
		INTERVAL_HIGHSCORE = 50,
		INTERVAL_SCROLL = 30,
		SCORE_LEVEL[] = { 500, 1000, 1500, 2000, Integer.MAX_VALUE },
		INTERVAL_GAME[] = { 500, 400, 300, 200, 100 },
		INTERVAL_DROP = 30,
		INTERVAL_STEP = 40,

		// States
	GAMEOVER = 0,
		HIGHSCORE = 1,
		GAME = 2,
		ABOUT = 3,
		PAUSED = 4,
		CHANGE_SENTENCE = 5,
		SCORE_GOOD = 10,
		SCORE_BAD = -5,
		HIT_THRESHOLD = 35,
		HIT_STEP = 5,
		MIN_WORDS = 5,
		MAX_WORDS = 15,
		MAX_ERRORS[] = { 5, 4, 3 },
		MAX_OVERALL_ERR[] = { 10, 5, 3 },
		CONS_GOOD = 5,
		NUM_SENTENCES = 10;

	public static final String STR_EXP[] = { "Nouns & verbs", "Major", "Minor", "All" };
	/*
	public  static final String ALL_FORMS_ORDER[] = {
				"conj",
				"pron",
				"art",
				"adp",
				"prp",
				"part",
				"adj",
				"n",
				"v",
				"prop",
				"adp",
				"adv",
				"num",
				"infm",
				"intj"
			};
			//*/
	
	//only for estoian
	/*
	public static final String ET_FORMS_ORDER[] =
	{
			"conj",
			"pron",
			"art",
			"adp",
			"part",
			"adj",
			"n",
			"v",
			"adv",
			"num",
			"infm",
			"intj" 
			};
			//*/
	
	protected static String ALL_FORMS[] =
		{
			"conj",
			"pron",
			"art",
			"prp",
			"part",
			"adj",
			"n",
			"v",
			"adv",
			"num",
			"infm",
			"intj" };

	protected WordFall applet;
	protected ImagePanel fallingrange, wall;
	protected FormBox[] forms;
	protected String form_str, custom;
	protected FallWord[] words;
	protected int activeform, wordindex;
	protected DropElementHandler dhandler;
	protected DropElement de = null;
	protected TextScrollHandler shandler;
	protected TextScrollComponent lastword;
	protected Vector sentences;
	protected int current_sentence;
	// Double Buffering
	protected Image buffImage;
	protected Graphics buffGraphics;
	// Settings
	protected String language;
	protected int score, state, overall_good, overall_bad, last_good;
	protected int word_class, overall_err, err, height, level, scorelevel;
	protected int num_words, stats_words[], stats_user[], stats_sentences;
	protected float overallhitrate;
	protected boolean dogameover, sounds;

	protected Timer tim;
	protected Random randgen;
	protected Vector infos, infocols;

	// *** GUI - stuff ***
	protected MenuItem startgame,
	    hs100a,
		hs100y,
		hs100m,
		hs10w,
		hs10d,
		mi_quit,
		mi_intro_en,
		mi_intro_da,
		mi_intro_fo,
		mi_about;
	protected CheckboxMenuItem mi_sounds, currentLanguage;
	protected Menu m_hs, m_lang, m_wc, m_level, m_help;
	protected HitClientPHP hc;
	protected HighScoreNameDialog inputname;
	protected CheckboxMenuGroup language_group, wc_group, level_group;
	protected FontMetrics fm = null;
	protected Panel formpanel = null, statuspanel = null;
	protected Label scorelabel, hitlabel;
	protected ProgressOMeter err_meter;
	protected Dimension dimension;

	public MainFrame(WordFall applet, String lang) {
		super(Dict.get("WordFall - Words falling out of the sky (Powered by VISL)"));
		language = lang;

		height = 0;
		dogameover = false;
		dimension = null;
		level = 1;
		scorelevel = 0;
		custom = null;
		word_class = 3;
		lastword = null;
		sentences = new Vector();
		current_sentence = -1;
		sounds = true;
		stats_words = null;
		stats_user = null;
		stats_sentences = 0;

		String scriptname = applet.getParameter("gameserver");
		if (scriptname == null)
			scriptname = "http://visl.sdu.dk/global/hitserver.php";
		hc =
			new HitClientPHP(
				scriptname,
				language,
				"WF" + word_class + "" + level);
		this.applet = applet;
		if (applet.getParameter("pattern") != null) {
			hc.setPattern(applet.getParameter("pattern"));
			custom = ":";
			for (int i = 0; i < ALL_FORMS.length; i++)
				custom += ALL_FORMS[i] + ":";
			word_class = 4;
			hc.setGame("WF" + word_class + "" + level);
		}
/*
		if (lang.equals("et")) {
		    DebugFrame.get().setShow(true);
			DebugFrame.get().pack();
			DebugFrame.get().setVisible(true);
			}
//*/
		Panel p = new Panel();
		p.setLayout(new BorderLayout());
		fallingrange = new ImagePanel();
		fallingrange.add(new ImageCanvas("background.jpg", ImageCanvas.TILED));
		fallingrange.resize(WIDTH, HEIGHT);
		wall = new ImagePanel();
		wall.add(new ImageCanvas("bricks.jpg", ImageCanvas.TILED));
		wall.resize(WIDTH, 0);   
		
		
		words = null;
		fallingrange.setForeground(Color.white);
		fallingrange.setBackground(Color.black);
		p.add("Center", fallingrange);
		p.add("North", wall);

		formpanel = new Panel();
		setupFormBoxes();
		p.add("South", formpanel);
		add("Center", p);

		statuspanel = new Panel();
		GridBagLayout gbl = new GridBagLayout();
		statuspanel.setLayout(gbl);
		GridBagConstraints c = new GridBagConstraints();
		c.weightx = 1;
		c.weighty = 0;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.BOTH;

		Label l = new Label(Dict.get("Score:"));
		l.setForeground(new Color(255, 100, 100));
		c.gridwidth = 1;
		gbl.setConstraints(l, c);
		statuspanel.add(l);
		scorelabel = new Label("0", Label.RIGHT);
		scorelabel.setForeground(new Color(255, 100, 100));
		c.gridwidth = GridBagConstraints.REMAINDER;
		gbl.setConstraints(scorelabel, c);
		statuspanel.add(scorelabel);

		l = new Label(Dict.get("Correct:"));
		l.setForeground(new Color(255, 100, 100));
		c.gridwidth = 1;
		gbl.setConstraints(l, c);
		statuspanel.add(l);
		hitlabel = new Label(" 100.0%", Label.RIGHT);
		hitlabel.setForeground(new Color(255, 100, 100));
		c.gridwidth = GridBagConstraints.REMAINDER;
		gbl.setConstraints(hitlabel, c);
		statuspanel.add(hitlabel);

		err_meter = new ProgressOMeter(0, MAX_ERRORS[level]);
		//err_meter.setCritical(0);
		err_meter.setCritical(MAX_ERRORS[level]);
		err_meter.setLevel(0);
		c.gridwidth = GridBagConstraints.REMAINDER;
		gbl.setConstraints(err_meter, c);
		statuspanel.add(err_meter);

		l = new Label(Dict.get("Penalty box"));
		l.setForeground(new Color(255, 100, 100));
		c.gridwidth = GridBagConstraints.REMAINDER;
		gbl.setConstraints(l, c);
		statuspanel.add(l);

		statuspanel.setForeground(new Color(255, 100, 100));
		statuspanel.setBackground(Color.black);
		statuspanel.setFont((Font) Miscellaneous.get("font.statuspanel"));
		add("East", statuspanel);

		buffImage = null;
		buffGraphics = null;
		randgen = new Random(System.currentTimeMillis());

		infos = new Vector();
		infocols = new Vector();
		addInfo("WordFall");
		addInfo("Press F2 to start game");

		state = GAMEOVER;

		inputname = new HighScoreNameDialog(this);
		inputname.hide();

		// **** MENU ****
		MenuBar mb = new MenuBar();
		Menu m = new Menu(Dict.get("Game"));
		CheckboxMenuItem cbmi = null;
		mb.add(m);
		startgame = new MenuItem(Dict.get("Start Game"));
		m.add(startgame);

		m_level = new Menu(Dict.get("Level"));
		level_group = new CheckboxMenuGroup();
		cbmi = new CheckboxMenuItem(Dict.get("Beginner"));
		level_group.add(cbmi);
		m_level.add(cbmi);
		cbmi = new CheckboxMenuItem(Dict.get("Intermediate"));
		level_group.add(cbmi);
		m_level.add(cbmi);
		cbmi = new CheckboxMenuItem(Dict.get("Expert"));
		level_group.add(cbmi);
		m_level.add(cbmi);
		m.add(m_level);
		level_group.setCurrentIndex(level);

		m_lang = new Menu(Dict.get("Language"));
		language_group = new CheckboxMenuGroup();
		String langs[][] = null;
		if (word_class == 4) {
			hc.setGame("WF00");
			langs = hc.getLanguages4Game();
			hc.setGame("WF" + word_class + "" + level);
		} else {
			langs = hc.getLanguages4Game();
		}
		for (int i = 0; i < langs.length; i++) {
			cbmi = new CheckboxMenuItem(langs[i][0] + " - " + langs[i][1]);
			language_group.add(cbmi);
			m_lang.add(cbmi);
			if (langs[i][0].equals(language)) {
				language_group.setCurrent(cbmi);
				currentLanguage = cbmi;
			}
		}
		m.add(m_lang);

		m_wc = new Menu(Dict.get("Word Classes"));
		wc_group = new CheckboxMenuGroup();
		cbmi = null;
		for (int i = 0; i < STR_EXP.length; i++) {
			cbmi = new CheckboxMenuItem(Dict.get(STR_EXP[i]));
			wc_group.add(cbmi);
			m_wc.add(cbmi);
		}
		cbmi = new CheckboxMenuItem(Dict.get("Custom"));
		wc_group.add(cbmi);
		m_wc.add(cbmi);

		wc_group.setCurrentIndex(word_class);
		m.add(m_wc);

		m_hs = new Menu(Dict.get("Highscore"));
		hs100a = new MenuItem(Dict.get("Top 100 all time"));
		hs100y = new MenuItem(Dict.get("Top 100 this year"));
		hs100m = new MenuItem(Dict.get("Top 100 this month"));
		hs10w = new MenuItem(Dict.get("Top 10 this week"));
		hs10d = new MenuItem(Dict.get("Top 10 today"));
		m_hs.add(hs100a);
		m_hs.add(hs100y);
		m_hs.add(hs100m);
		m_hs.add(hs10w);
		m_hs.add(hs10d);
		m.add(m_hs);
		mi_quit = new MenuItem(Dict.get("Quit"));
		m.add(mi_quit);

		Menu m3 = new Menu(Dict.get("Settings"));
		mi_sounds = new CheckboxMenuItem(Dict.get("Sound Effects"));
		mi_sounds.setState(sounds);
		m3.add(mi_sounds);
		mb.add(m3);

		Menu m2 = new Menu(Dict.get("Help"));
		m_help = new Menu(Dict.get("Introduction"));
		m2.add(m_help);
		mi_intro_en = new MenuItem(Dict.get("en"));
		mi_intro_da = new MenuItem(Dict.get("da"));
		mi_intro_fo = new MenuItem(Dict.get("fo"));
		m_help.add(mi_intro_en);
		m_help.add(mi_intro_da);
		m_help.add(mi_intro_fo);
		mi_about = new MenuItem(Dict.get("About..."));
		m2.add(mi_about);
		mb.add(m2);

		setMenuBar(mb);
		pack();
		setResizable(true);

		dhandler = new DropElementHandler(20);
		shandler = new TextScrollHandler();
		shandler.colorChange(true);

		tim = new Timer(INTERVAL_GAME[scorelevel]);
		tim.addListener(this);
		tim.start();
	}
	
	public boolean handleEvent(Event e) {
		if ((e.id == Event.MOUSE_ENTER && e.target == this)
			|| (e.id == Event.GOT_FOCUS && e.target == fallingrange)) {
			update(getGraphics());
		}
		if (dogameover && e.target == this && e.id == 0 && e.arg == null) {
			dogameover = false;
			gameOver();
		}
		if (e.id == Event.WINDOW_DESTROY) {
			doQuit();
			return true;
		} else if (
			e.id == Event.WINDOW_MOVED
				|| ((e.id == Event.MOUSE_DRAG || e.id == Event.MOUSE_MOVE)
					&& e.target == this)) {
			synchronized (fallingrange) {
				buffImage = null;
				buffGraphics = null;
			}
			return true;
		} else if (state == PAUSED && (e.id == Event.KEY_PRESS)) {
			clearInfos();
			for (int i = 0; i < words.length; i++)
				words[i].show(true);
			state = GAME;
			update(fallingrange.getGraphics());
		} else if (state == GAME && e.id == Event.KEY_PRESS && e.key == 'p') {
			state = PAUSED;
			for (int i = 0; i < words.length; i++)
				words[i].hide();
			addInfo(Dict.get("Game Paused!"));
			update(fallingrange.getGraphics());
		} else if (
			(state != GAME && state != CHANGE_SENTENCE)
				&& ((e.target == startgame && e.id == Event.ACTION_EVENT)
					|| (e.id == Event.KEY_ACTION && e.key == Event.F2))) {
			startGame();
		} else if (
			((state == GAME || state == CHANGE_SENTENCE)
				&& e.target == startgame
				&& e.id == Event.ACTION_EVENT)) {
			dogameover = true;
		} else if (
			e.id == Event.KEY_ACTION
				&& de != null
				&& (e.key == Event.LEFT || e.key == Event.RIGHT)) {
			forms[activeform].reset();
			if (e.key == Event.LEFT) {
				if (activeform > 0)
					activeform--;
				else
					activeform = forms.length - 1;
			} else if (e.key == Event.RIGHT) {
				if (activeform < forms.length - 1)
					activeform++;
				else
					activeform = 0;
			}
			Dimension d = de.size();
			forms[activeform].resize(d);
			formpanel.validate();
			Rectangle r = forms[activeform].bounds();
			Point p = de.getPosition();
			de.setPosition(r.x + r.width / 2, p.y);
			update(fallingrange.getGraphics());
			return true;
		} else if (e.id == Event.KEY_ACTION && e.key == Event.DOWN) {
			synchronized (tim) {
				tim.setInterval(INTERVAL_DROP);
			}
		} else if (e.id == Event.KEY_RELEASE) {
		} else if (state == GAMEOVER && e.id == Event.MOUSE_DOWN) {
			TextScrollComponent tsc = null;
			// EXTREMELY HACKY!!!
			Rectangle r = fallingrange.getParent().bounds();
			for (int i = 0; i < infos.size() && tsc == null; i++) {
				if (infos.elementAt(i) instanceof TextScrollComponent) {
					TextScrollComponent tsc2 =
						(TextScrollComponent) infos.elementAt(i);
					if (tsc2.isHit(e.x - r.x, e.y - r.y)) {
						tsc = tsc2;
					}
				}
			}
			//System.out.println("tsc = " + tsc);
			if (tsc != null) {
				if (tsc.text.equals(Dict.get("Yes"))) {
					word_class = 4;
					wc_group.setCurrentIndex(word_class);
					hc.setGame("WF" + word_class + "" + level);
					setupFormBoxes();
					startGame();
				} else if (tsc.text.equals(Dict.get("No"))) {
					doHighScores("T10D");
				}
			}
			return true;
		} else if (
			state == GAMEOVER
				&& custom != null
				&& e.id == Event.KEY_PRESS
				&& e.key == 'y') {
			word_class = 4;
			wc_group.setCurrentIndex(word_class);
			hc.setGame("WF" + word_class + "" + level);
			setupFormBoxes();
			startGame();
			return true;
		} else if (
			state == GAMEOVER
				&& custom != null
				&& e.id == Event.KEY_PRESS
				&& e.key == 'n') {
			doHighScores("T10D");
			return true;
		} else if (
			e.id == Event.ACTION_EVENT && e.target instanceof MenuItem) {
			if (e.target == hs100a
			    || e.target == hs100y
				|| e.target == hs100m
				|| e.target == hs10w
				|| e.target == hs10d) {
				String s = null;
				if (e.target == hs100y)
					s = "T100Y";
				else if (e.target == hs100a)
				    s = "T100A";
				else if (e.target == hs100m)
					s = "T100M";
				else if (e.target == hs10w)
					s = "T10W";
				else if (e.target == hs10d)
					s = "T10D";
				doHighScores(s);
			} else if (e.target == mi_quit) {
				doQuit();
			} else if (e.target == mi_intro_en) {
				try {
					applet.getAppletContext().showDocument(
						new URL(
							(String) Miscellaneous.get("url.wordfall.help")),
						"Help");
				} catch (MalformedURLException x) {
					System.out.println(
						"Could not show ["
							+ Miscellaneous.get("url.wordfall.help")
							+ "]: "
							+ x);
				}
			} else if (e.target == mi_intro_da) {
				try {
					applet.getAppletContext().showDocument(
						new URL(
							(String) Miscellaneous.get("url.wordfall.da.help")),
						"Help");
				} catch (MalformedURLException x) {
					System.out.println(
						"Could not show ["
							+ Miscellaneous.get("url.wordfall.da.help")
							+ "]: "
							+ x);
				}
			} else if (e.target == mi_intro_fo) {
				try {
					applet.getAppletContext().showDocument(
						new URL(
							(String) Miscellaneous.get("url.wordfall.fo.help")),
						"Help");
				} catch (MalformedURLException x) {
					System.out.println(
						"Could not show ["
							+ Miscellaneous.get("url.wordfall.fo.help")
							+ "]: "
							+ x);
				}
			} else if (e.target == mi_about && state == GAMEOVER) {
				state = ABOUT;
				shandler.clear();
				tim.setInterval(INTERVAL_HIGHSCORE);
				Dimension d = fallingrange.size();
				shandler.setSize(d.width, d.height);
				TextScrollComponent tsc = null;
				int x = 0, y = d.height;
				float speed = -0.03f;
				String s = null;

				s = Dict.get("WordFall");
				tsc = new TextScrollComponent(s, 0, x, y, "");
				tsc.vy = speed;
				tsc.x = (d.width - tsc.width) / 2;
				shandler.add(tsc);
				y += tsc.height * 2;

				s = Dict.get("VISL - Visual Interactive Syntax Learning");
				tsc = new TextScrollComponent(s, 0, x, y, "");
				tsc.vy = speed;
				tsc.x = (d.width - tsc.width) / 2;
				shandler.add(tsc);
				y += tsc.height * 2;

				s = Dict.get("May 2001");
				tsc = new TextScrollComponent(s, 0, x, y, "");
				tsc.vy = speed;
				tsc.x = (d.width - tsc.width) / 2;
				shandler.add(tsc);
				y += tsc.height * 2;

				s = Dict.get("Rasmus Lock Larsen");
				tsc = new TextScrollComponent(s, 0, x, y, "");
				tsc.vy = speed;
				tsc.x = (d.width - tsc.width) / 2;
				shandler.add(tsc);
				y += tsc.height * 2;

				s = Dict.get("<rasmus@mip.sdu.dk>");
				tsc = new TextScrollComponent(s, 0, x, y, "");
				tsc.vy = speed;
				tsc.x = (d.width - tsc.width) / 2;
				shandler.add(tsc);
			} else if (e.target instanceof CheckboxMenuItem) {
				CheckboxMenuItem cbmi = (CheckboxMenuItem) e.target;
				if (language_group.contains(cbmi)) {
					//Change in the language settings
					String oldLanguage = language;
					String s = cbmi.getLabel();
					language = s.substring(0, s.indexOf(" - "));
					if (applet.isJava1_3Compatible(language)) {
						language_group.setCurrent(cbmi);
						currentLanguage = cbmi;
						hc.setLanguage(language);
					} else {
						language_group.setCurrent(currentLanguage);
						language = oldLanguage;
						YesNoDialog warning =
							new YesNoDialog(
								this,
								Dict.get("Warning!"),
								Dict.get(
									"Because a seperate plug-in is required to handle the font \nin this language, you can not change to this language in this way.\nYou have to close the game and choose \nthe new language from the homepage to use it."));
						warning.show();
					}
					setupFormBoxes();
				} else if (wc_group.contains(cbmi)) {
					int oldclass = wc_group.getCurrentIndex();
					wc_group.setCurrent(cbmi);
					word_class = wc_group.getCurrentIndex();

					if (word_class == 4) {
						// Show CustomDialogBox
						// HACKY!!
						CustomDialog cd = new CustomDialog(this, ALL_FORMS, "");
						cd.setFont((Font) Miscellaneous.get("font.custom"));
						cd.pack();
						cd.show();
						if (cd.getForms() != null) {
							custom = cd.getForms();
						} else {
							word_class = oldclass;
						}
						cd.dispose();
					}
					wc_group.setCurrentIndex(word_class);
					hc.setGame("WF" + word_class + "" + level);
					setupFormBoxes();
				} else if (level_group.contains(cbmi)) {
					level_group.setCurrent(cbmi);
					level = level_group.getCurrentIndex();
					hc.setGame("WF" + word_class + "" + level);
				} else if (cbmi == mi_sounds) {
					sounds = !sounds;
					cbmi.setState(sounds);
				}
				return true;
			}
		}
		return false;
	}
	public void update(Graphics g) {
		paint(g);
	}
	public void paint(Graphics g) {
		if (state != GAMEOVER)
			updateStatus();
		synchronized (fallingrange) {
			if (buffImage == null || buffGraphics == null) {
				buffImage = null;
				buffGraphics = null;
				TextScrollComponent.setFontMetrics(
					getFontMetrics((Font) Miscellaneous.get("scrollfont")));
				fm = TextScrollComponent.getFontMetrics();
				dimension = fallingrange.size();
				height = dimension.height + wall.size().height;
				dhandler.setStepSize(height / 25);
				buffImage = createImage(dimension.width, dimension.height);
				if (buffImage != null)
					buffGraphics = buffImage.getGraphics();
				else
					buffGraphics = null;
			}
			// Paint background
			//g = shootingrange.getGraphics();
			if (buffGraphics != null) {
				fallingrange.paint(buffGraphics);
				buffGraphics.setColor(Color.gray);
				switch (state) {
					case ABOUT :
					case HIGHSCORE :
					case CHANGE_SENTENCE :
						buffGraphics.setFont(
							(Font) Miscellaneous.get("dropfont"));
						shandler.updateGraphics(buffGraphics, null);
						break;
					case GAME :
						buffGraphics.setFont(
							(Font) Miscellaneous.get("dropfont"));
						dhandler.updateGraphics(buffGraphics, null);
						break;
				}
				updateInfos();
				if (g != null && buffImage != null)
					g.drawImage(buffImage, 0, 0, this);
			}
		}
	}
	
	
	
	
	
	public void tick() {
		if (dogameover && state != GAMEOVER) {
			wall.resize(0, 0);
			clearInfos();
			addInfo("G A M E   O V E R!");
			state = GAMEOVER;
			validate();
			this.postEvent(new Event(this, 0, null));
		}
		switch (state) {
			case PAUSED :
			case GAMEOVER :
				shandler.tick();
				break;
			case CHANGE_SENTENCE :
				shandler.tick();
				for (int i = 0; i < infocols.size(); i++) {
					Color col = (Color) infocols.elementAt(i);
					float hsb[] = new float[3];
					Color.RGBtoHSB(
						col.getRed(),
						col.getGreen(),
						col.getBlue(),
						hsb);
					if (hsb[0] < 1)
						hsb[0] += 0.01f;
					else
						hsb[0] = 0;
					infocols.setElementAt(
						new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2])),
						i);
				}
				if (lastword != null
					&& lastword.x <= words[words.length - 1].bounds().x) {
					shandler.clear();
					for (int i = 0; i < words.length; i++)
						words[i].show();
					state = GAME;
					shandler.colorChange(true);
					nextWord();
				}
				break;
			case HIGHSCORE :
			case ABOUT :
				shandler.tick();
				break;
			case GAME :
				dhandler.tick();
				if (dhandler.getPosition().y > fallingrange.size().height) {
					if (words[wordindex]
						.getForm()
						.equals(forms[activeform].getForm())) {
						words[wordindex].setForeground(Colors.get("word.ok"));
						stats_words[forms[activeform].getIndex()]++;
						stats_user[forms[activeform].getIndex()]++;
						score += SCORE_GOOD;
						if (sounds)
							Sounds.playSound(0);
						overall_good++;
					} else {
						words[wordindex].setForeground(
							Colors.get("word.fault"));
						score += SCORE_BAD;
						if (sounds)
							Sounds.playSound(1);
						overall_bad++;
						err++;
						err_meter.setLevel(err);
						FormBox fb = null;
						for (int i = 0; i < forms.length && fb == null; i++) {
							if (forms[i]
								.getForm()
								.equals(words[wordindex].getForm())) {
								fb = forms[i];
							}
						}
						if (fb != null) {
							fb.doBlink();
							stats_words[fb.getIndex()]++;
						} else
							System.out.println("No fb was found!");
						if (err >= MAX_ERRORS[level]) {
							if (height <= 0)
								height =
									wall.size().height
										+ fallingrange.size().height;
							err_meter.setLevel(0);
							err = 0;
							overall_err++;
							wall.resize(
								WIDTH,
								height
									* overall_err
									/ (MAX_OVERALL_ERR[level] + 1));
							validate();
						}
					}
					if (score >= SCORE_LEVEL[scorelevel]) {
						scorelevel++;
					} else if (
						scorelevel > 0
							&& score < SCORE_LEVEL[scorelevel - 1]) {
						scorelevel--;
					}
					overallhitrate =
						(overall_good + overall_bad > 0
							? 100.0f
								* overall_good
								/ (overall_good + overall_bad)
							: 0);
					overallhitrate = ((int) overallhitrate * 10) / 10;
					dhandler.clear();
					if (overall_err <= MAX_OVERALL_ERR[level])
						nextWord();
					else
						dogameover = true;
				}
				break;
		}
		update(fallingrange.getGraphics());
	}
	protected void updateStatus() {
		scorelabel.setText(score + " ");
		hitlabel.setText(overallhitrate + "%");
		scorelabel.repaint();
		hitlabel.repaint();
	}
	protected void gameOver() {
		dhandler.clear();
		wall.resize(0, 0);
		validate();
		clearInfos();
		addInfo(Dict.get("G A M E   O V E R !"));
		repaint();
		//update(fallingrange.getGraphics());
		if (word_class != 4) {
			int pos = hc.getPosition(score, overallhitrate);
			if (pos != -1 && score > 0) {
				addInfo(Dict.get("Please supply your name and location"));
				inputname.show(score, pos + 1);
			}
			try {
				while (inputname.isVisible())
					Thread.sleep(100);
			} catch (InterruptedException x) {
			}
			if (!inputname.getName().equals("")) {
				String name = inputname.getName();
				if (name.length() > 12)
					name = name.substring(0, 12);
				String country = inputname.getCountry();
				if (country.length() > 3)
					country = country.substring(0, 3);
				hc.putHighScore(name, country, score, overallhitrate);
			}
			//doHighScores("T10D");
		}
		dhandler.clear();
		de = null;
		// Show Stats
		shandler.clear();
		clearInfos();

		// **** Statistics ****
		TextScrollComponent tsc = null;
		String s = null;
		int stepy = dimension.height / 10;
		int y = stepy;
		int x = 0;
		int stepx = dimension.width / (ALL_FORMS.length + 2);
		Color col = new Color(255, 255, 100);
		s = Dict.get("_Statistics_");
		x = (dimension.width - fm.stringWidth(s)) / 2;
		tsc = new TextScrollComponent(s, 0, x, y, "");
		tsc.color = col;
		addInfo(tsc);

		x = stepx / 4;
		y = 3 * stepy;
		s = Dict.get("All");
		tsc = new TextScrollComponent(s, 0, x, y, "");
		tsc.color = col;
		addInfo(tsc);
		y += stepy;
		s = Dict.get("Correct");
		tsc = new TextScrollComponent(s, 0, x, y, "");
		tsc.color = col;
		addInfo(tsc);
		y += stepy;
		s = Dict.get("Pct.");
		tsc = new TextScrollComponent(s, 0, x, y, "");
		tsc.color = col;
		addInfo(tsc);
		y += stepy;

		x = 2 * stepx;
		custom = ":";
		int custom_cnt = 0;
		for (int i = 0; i < ALL_FORMS.length; i++) {
			y = 2 * stepy;

			s = ALL_FORMS[i]; // Short version of wordclasses
			tsc = new TextScrollComponent(s, 0, x, y, "");
			tsc.color = col;
			addInfo(tsc);

			boolean activeform =
				form_str.indexOf(":" + ALL_FORMS[i] + ":") != -1;

			y += stepy;
			s = (activeform ? "" + stats_words[i] : "-");
			tsc = new TextScrollComponent(s, 0, x, y, "");
			tsc.color = col;
			addInfo(tsc);

			y += stepy;
			s = (activeform ? "" + stats_user[i] : "-");
			tsc = new TextScrollComponent(s, 0, x, y, "");
			tsc.color = col;
			addInfo(tsc);

			y += stepy;
			int q =
				(stats_words[i] != 0
					? (stats_user[i] * 100) / stats_words[i]
					: 100);
			s = (activeform && stats_words[i] != 0 ? "" + q + "%" : "-");
			tsc = new TextScrollComponent(s, 0, x, y, "");
			if (q < 75) {
				tsc.color = Color.red;
				custom += ALL_FORMS[i] + ":";
				custom_cnt++;
			} else if (stats_words[i] != 0)
				tsc.color = Color.green;
			else {
				tsc.color = col;
				if (activeform && stats_words[i] == 0) {
					custom += ALL_FORMS[i] + ":";
					custom_cnt++;
				}
			}
			addInfo(tsc);

			x += stepx;
		}
		if (custom_cnt >= 2) {
			y += 2 * stepy;
			s =
				Dict.get(
					"Do you want a custom-made game to improve your performance?");
			x = (dimension.width - fm.stringWidth(s)) / 2;
			tsc = new TextScrollComponent(s, 0, x, y, "");
			tsc.color = col;
			addInfo(tsc);

			y += stepy;
			x = dimension.width / 3;
			s = Dict.get("Yes");
			tsc = new TextScrollComponent(s, 0, x, y, "");
			tsc.color = Color.green;
			addInfo(tsc);

			x = 2 * dimension.width / 3;
			s = Dict.get("No");
			tsc = new TextScrollComponent(s, 0, x, y, "");
			tsc.color = Color.red;
			addInfo(tsc);
		} else {
			if (applet.getParameter("pattern") == null)
				custom = null;
			word_class = 3;
			wc_group.setCurrentIndex(word_class);
			hc.setGame("WF" + word_class + "" + level);
			setupFormBoxes();
		}

		fallingrange.removeAll();
		words = null;
		lastword = null;
		// Enable menus again
		m_hs.enable();
		m_lang.enable();
		m_wc.enable();
		m_level.enable();
		startgame.setLabel(Dict.get("Start Game"));
		repaint();
	}
	protected void printCenter(String s, int offsy, Color col) {
		Graphics g = buffGraphics;
		Dimension d = fallingrange.size();
		g.setFont(fm.getFont());
		g.setColor(Color.black);
		g.drawString(
			s,
			(d.width - fm.stringWidth(s)) / 2 + 2,
			d.height / 2 + offsy + 2);
		g.setColor(col);
		g.drawString(
			s,
			(d.width - fm.stringWidth(s)) / 2,
			d.height / 2 + offsy);
	}
	protected void doHighScores(String s) {
		clearInfos();
		dhandler.clear();
		// Make sure we are not in a custom mode
		if (word_class == 4) {
			addInfo(Dict.get("No Highscores available for custom made games"));
			update(fallingrange.getGraphics());
			return;
		}
		// Change speed of timer
		tim.setInterval(INTERVAL_HIGHSCORE);
		state = HIGHSCORE;
		if (word_class == 4)
			return;
		Dimension d = fallingrange.size();
		addInfo("Retrieving HighScore!");
		update(getGraphics());
		shandler.clear();
		shandler.setSize(d.width, d.height);
		shandler.colorChange(true);

		fm = getFontMetrics((Font) Miscellaneous.get("dropfont"));
		Vector v = hc.getHighScore(s);
		int sX = d.width / 49;
		int sH = 3 * fm.getHeight() / 2;
		int x0 = 0;
		int x1 = 5 * sX;
		int x2 = x1 + 8 * sX;
		int x3 = x2 + 6 * sX;
		int x4 = x3 + 13 * sX;
		int x5 = x4 + 3 * sX;
		int x6 = x5 + 9 * sX;
		double speed = (s.equals("T100A") || s.equals("T100Y") || s.equals("T100M") ? -0.08 : -0.04);

		HighScoreItem hsi = null;
		int x = 100;
		int y = d.height - 20;
		Vector list = new Vector();
		s = Dict.get(s);
		x = (d.width - fm.stringWidth(s)) / 2;
		TextScrollComponent tsc = new TextScrollComponent(s, 0, x, y, "");
		tsc.vy = speed;
		list.addElement(tsc);
		y += sH;
		s = Dict.get("Place");
		tsc = new TextScrollComponent(s, 0, x0, y, "");
		tsc.vy = speed;
		list.addElement(tsc);
		// Right justified
		s = Dict.get("Score");
		tsc = new TextScrollComponent(s, 0, x2 - sX - fm.stringWidth(s), y, "");
		tsc.vy = speed;
		list.addElement(tsc);
		// Right justified
		s = Dict.get("Hit");
		tsc = new TextScrollComponent(s, 0, x3 - sX - fm.stringWidth(s), y, "");
		tsc.vy = speed;
		list.addElement(tsc);
		s = Dict.get("Name");
		tsc = new TextScrollComponent(s, 0, x3, y, "");
		tsc.vy = speed;
		list.addElement(tsc);
		// Right justified
		s = Dict.get("Country");
		tsc = new TextScrollComponent(s, 0, x5 - sX - fm.stringWidth(s), y, "");
		tsc.vy = speed;
		list.addElement(tsc);
		// Right justified
		s = Dict.get("Date");
		tsc = new TextScrollComponent(s, 0, x6 - sX - fm.stringWidth(s), y, "");
		tsc.vy = speed;
		list.addElement(tsc);
		s = Dict.get("Time");
		tsc =
			new TextScrollComponent(
				s,
				0,
				x6 + 5 * sX - fm.stringWidth(s),
				y,
				"");
		tsc.vy = speed;
		list.addElement(tsc);
		y += sH;
		for (int i = 0; i < v.size(); i++) {
			hsi = (HighScoreItem) v.elementAt(i);
			// place
			s = "" + (i + 1) + ".";
			// Right justified
			tsc =
				new TextScrollComponent(
					s,
					0,
					x1 - sX - fm.stringWidth(s),
					y,
					"");
			tsc.vy = speed;
			list.addElement(tsc);
			// score
			s = "" + hsi.score;
			// Right justified
			tsc =
				new TextScrollComponent(
					s,
					0,
					x2 - sX - fm.stringWidth(s),
					y,
					"");
			tsc.vy = speed;
			list.addElement(tsc);
			// hitrate
			s = "" + hsi.hitrate + "%";
			// Right justified
			tsc =
				new TextScrollComponent(
					s,
					0,
					x3 - sX - fm.stringWidth(s),
					y,
					"");
			tsc.vy = speed;
			list.addElement(tsc);
			// name
			s = hsi.name;
			tsc = new TextScrollComponent(s, 0, x3, y, "");
			tsc.vy = speed;
			list.addElement(tsc);
			// country
			s = hsi.country;
			tsc = new TextScrollComponent(s, 0, x4, y, "");
			tsc.vy = speed;
			list.addElement(tsc);
			// date
			// Right justified
			s =
				hsi.date.getDate()
					+ "-"
					+ (hsi.date.getMonth() + 1)
					+ "-"
					+ (hsi.date.getYear() + 1900);
			tsc =
				new TextScrollComponent(
					s,
					0,
					x6 - sX - fm.stringWidth(s),
					y,
					"");
			tsc.vy = speed;
			list.addElement(tsc);
			// time
			// Right justified
			int minutes = hsi.date.getMinutes();
			s =
				hsi.date.getHours()
					+ ":"
					+ (minutes > 9 ? "" + minutes : "0" + minutes);
			tsc =
				new TextScrollComponent(
					s,
					0,
					x6 + 5 * sX - fm.stringWidth(s),
					y,
					"");
			tsc.vy = speed;
			list.addElement(tsc);
			y += sH;
		}
		shandler.tick();
		for (Enumeration en = list.elements(); en.hasMoreElements();) {
			shandler.add((TextScrollComponent) en.nextElement());
		}
		v.removeAllElements();
		clearInfos();
	}
	protected void clearInfos() {
		synchronized (infos) {
			infos.removeAllElements();
		}
	}
	protected void addInfo(Object o) {
		synchronized (infos) {
			infos.addElement(o);
		}
	}
	protected void updateInfos() {
		synchronized (infos) {
			if (infos.size() != 0) {
				buffGraphics.setColor(Color.red);
				int H = fm.getHeight();
				int n = infos.size();
				for (int i = 0; i < n; i++)
					if (infos.elementAt(i) instanceof String) {
						printCenter(
							(String) infos.elementAt(i),
							(i - n / 2) * 2 * H,
							(infocols.size() <= i
								? Color.white
								: (Color) infocols.elementAt(i)));
					} else if (
						infos.elementAt(i) instanceof TextScrollComponent) {
						TextScrollComponent tsc =
							(TextScrollComponent) infos.elementAt(i);
						if (buffGraphics != null) {
							fm = TextScrollComponent.getFontMetrics();
							buffGraphics.setFont(fm.getFont());
							buffGraphics.setColor(tsc.color);
							if (tsc.text.startsWith("_")
								&& tsc.text.endsWith("_")) {
								String s =
									tsc.text.substring(
										1,
										tsc.text.length() - 1);
								buffGraphics.drawString(
									s,
									(int) tsc.x,
									(int) tsc.y);
								buffGraphics.drawLine(
									(int) tsc.x,
									(int) (tsc.y + H / 2),
									(int) tsc.x + fm.stringWidth(s),
									(int) (tsc.y + H / 2));

							} else {
								buffGraphics.drawString(
									tsc.text,
									(int) tsc.x,
									(int) tsc.y);
							}
						}
					}
			}
		}
	}
	protected void doQuit() {
		//stopTimers();
		tim.stop();
		//tim2.stop();
		dispose();
		applet.quit(Dict.get("Good-bye from WordFall!"), null);

	}
	protected void startGame() {
		// Disable menus again
		//System.out.println("Starting new game [custom=" + custom + "]");
		m_hs.disable();
		m_lang.disable();
		m_wc.disable();
		m_level.disable();
		startgame.setLabel(Dict.get("Stop game"));
		shandler.clear();
		score = 0;
		overallhitrate = 0;
		overall_good = 0;
		overall_bad = 0;
		overall_err = 0;
		last_good = 0;
		err = 0;
		height = 0;
		scorelevel = 0;
		num_words = -1;
		current_sentence = -1; // Force download of new sentences
		de = null;
		shandler.clear();
		clearInfos();
		words = null;
		err_meter.setLevel(0);
		for (int i = 0; i < stats_words.length; i++) {
			stats_words[i] = 0;
			stats_user[i] = 0;
		}
		state = GAME;
		nextSentence();
	}
	protected void nextSentence() {
		state = PAUSED;
		fallingrange.removeAll();
		Sentence s = null;
		int okcnt = 0;
		int retries = 0;
		Vector v = new Vector();
		// Save the current sentence
		float speed = -0.16f;
		tim.setInterval(INTERVAL_SCROLL);
		shandler.clear();
		shandler.colorChange(false);
		shandler.setSize(dimension.width, dimension.height);
		int sH = TextScrollComponent.getFontMetrics().getHeight();
		int err = (num_words > 0 ? num_words - (overall_good - last_good) : -1);
		if (err == 0) {
			if (sounds)
				Sounds.playSound(2);
			addInfo(Dict.get("Perfect sentence!"));
			addInfo(Dict.get("Bonus : ") + num_words * SCORE_GOOD);
			infocols.addElement(new Color(255, 0, 0));
			infocols.addElement(new Color(255, 0, 0));
			score += num_words * SCORE_GOOD;
		} else if (err > 0) {
			addInfo(Dict.get("No Bonus!"));
			infocols.addElement(new Color(255, 0, 0));
		}
		if (words != null) {
			for (int i = 0; i < words.length; i++) {
				Rectangle r = words[i].bounds();
				TextScrollComponent tsc =
					new TextScrollComponent(
						words[i].getText(),
						speed,
						r.x,
						r.y + sH,
						"");
				tsc.color = words[i].getForeground();
				v.addElement(tsc);
			}
		}
		int rejected = 0;
		if (current_sentence < 0 || current_sentence >= sentences.size() - 1) {
			sentences =
				hc.getRandomSentencesWithForm(
					form_str,
					MIN_WORDS,
					MAX_WORDS,
					NUM_SENTENCES);
			current_sentence = 0;
		} else
			current_sentence++;
		if (sentences.size() > 0) {
			s = (Sentence) sentences.elementAt(current_sentence);
		} else {
			s = new Sentence();
			addInfo(Dict.get("No sentences matched the criteria"));
			if (applet.getParameter("pattern") != null) {
				String pat = applet.getParameter("pattern");
				addInfo(
					Dict.get("Perhaps the pattern ")
						+ pat
						+ Dict.get(" is too restrictive."));
			}

		}
		//DebugFrame.get().addInfo(s.toString());
		words = new FallWord[s.size()];
		stats_sentences++;
		num_words = 0;
		if (!language.equals("ar")) {
		    for (int i = 0; i < words.length; i++) {
			Word word = s.getWord(i);
			// 20021002 - Rasmus, hc has a new method
			// word.text = encodeString(word.text);
			//System.out.println("-"+ i +":" + word);
			//word.text = hc.newString(word.text);
			//System.out.println("+"+ i +":" + word);
			words[i] = new FallWord(word);
			
			fallingrange.add(words[i]);
			if (form_str.indexOf(":" + word.form + ":") == -1)
			    words[i].setFalled(true);
			else
			    num_words++;
		    }
		}else{
		    for (int i = words.length-1; i >= 0 ; i--) {
			Word word = s.getWord(i);
			words[i] = new FallWord(word);
			
			fallingrange.add(words[i]);
			if (form_str.indexOf(":" + word.form + ":") == -1)
			    words[i].setFalled(true);
			else
			    num_words++;
		    }
		}
		fallingrange.validate();
		formpanel.validate();
		
		for (int i = 0; i < words.length; i++) {
		    Rectangle r = words[i].bounds();
		    words[i].hide();
		    TextScrollComponent tsc =
			new TextScrollComponent(
						words[i].getText(),
						speed,
						dimension.width + r.x,
						r.y + sH,
						"");
		    v.addElement(tsc);
		}

		shandler.tick();
		lastword = (TextScrollComponent) v.lastElement();
		for (int i = 0; i < v.size(); i++) {
			shandler.add((TextScrollComponent) v.elementAt(i));
		}
		last_good = overall_good;
		state = CHANGE_SENTENCE;
	}
	protected void nextWord() {
		clearInfos();
		state = GAME;
		tim.setInterval(INTERVAL_GAME[scorelevel]);
		int newindex = (int) (randgen.nextDouble() * (words.length - 1));
		int cnt = 1;
		while (cnt < words.length && words[newindex].hasFalled()) {
			newindex = ((newindex + 1) % words.length);
			cnt++;
		}
		if (words[newindex].hasFalled()) {
			if (overall_err <= MAX_OVERALL_ERR[level])
				nextSentence();
			else
				dogameover = true;
			return;
		}
		wordindex = newindex;
		forms[activeform].reset();
		formpanel.validate();

		de = new FallWord(words[newindex]);
		activeform = 0;
		Rectangle r = forms[activeform].bounds();
		Rectangle r2 = words[newindex].bounds();
		while ((r.x + r.width) < (r2.x + r2.width / 2)
			&& activeform < forms.length - 1) {
			activeform++;
			r = forms[activeform].bounds();
		}
		forms[activeform].resize(de.size());
		//de.setPosition(r.x + r.width/2, words[newindex].bounds().y);
		de.setPosition(r2.x + r2.width / 2, r2.y);
		formpanel.validate();
		dhandler.setDropElement(de);
	}
	public void setupFormBoxes() {
		formpanel.removeAll();
		Vector v = hc.getWordClasses("all");
		ALL_FORMS = new String[v.size()];
		stats_words = new int[v.size()];
		stats_user = new int[v.size()];
		stats_sentences = 0;
		int cnt = 0;
		if (!language.equals("ar")) {
		/*
			if(false && language.equals("et")) {
				System.out.println("in et-mode");
				//case et, we have some different wordclasses (adp <-> prd )	
				for (int i = 0; i < ET_FORMS_ORDER.length; i++) {
					if (v.contains(ET_FORMS_ORDER[i])) {
						System.out.println("ET_FORMS_ORDER[" + i +"] = " + ET_FORMS_ORDER[i]  );
						ALL_FORMS[cnt++] = ET_FORMS_ORDER[i];
					}
				}
			} else {
				for (int i = 0; i < ALL_FORMS_ORDER.length; i++) {
					//System.out.println(ALL_FORMS_ORDER[i] + " " + i);
					if (v.contains(ALL_FORMS_ORDER[i])) {
						ALL_FORMS[cnt++] = ALL_FORMS_ORDER[i];
						//System.out.println(ALL_FORMS[cnt-1] + " " + cnt);
					}
				}
			}
			//*/
				for (Enumeration e = v.elements(); e.hasMoreElements();) {
					ALL_FORMS[cnt++] = (String) e.nextElement();
				}
			
			} else {
				for (Enumeration e = v.elements(); e.hasMoreElements();) {
					ALL_FORMS[cnt++] = (String) e.nextElement();
				} //The Formbox array is reverse
				String tmp = "";
				for (int i = 0; i < ALL_FORMS.length / 2; i++) {
					tmp = ALL_FORMS[i];
					ALL_FORMS[i] = ALL_FORMS[ALL_FORMS.length - 1 - i];
					ALL_FORMS[ALL_FORMS.length - 1 - i] = tmp;
				}
			}
			
			GridBagLayout gbl = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();
			c.weightx = 1;
			c.weighty = 1.0;
			c.anchor = GridBagConstraints.WEST;
			c.fill = GridBagConstraints.BOTH;
			formpanel.setLayout(gbl);
			
			switch (word_class) {
			case 0 :
				v = hc.getWordClasses("nv");
				break;
			case 1 :
				v = hc.getWordClasses("major");
				break;
			case 2 :
				v = hc.getWordClasses("minor");
				break;
			case 3 :
				v = hc.getWordClasses("all");
				break;
			}
			if (word_class != 4) {
				forms = new FormBox[v.size()];
				form_str = ":";
				for (int i = 0; i < v.size(); i++)
					form_str += ":" + (String) v.elementAt(i);
				form_str += ":";
			} else {
				form_str = custom;
				int count = 0, ix = 0, jx = 0;
				while ((jx = form_str.indexOf(':', ix)) != -1) {
					ix = jx + 1;
					count++;
				}
				count--;
				forms = new FormBox[count];
			}
			
			cnt = 0;
			for (int i = 0; i < ALL_FORMS.length; i++) {
			    if (ALL_FORMS[i] == null || ALL_FORMS[i].trim().equals("")) {
			        ALL_FORMS[i] = "xxx";
			    }
				FormBox fb = new FormBox(ALL_FORMS[i], i);
				gbl.setConstraints(fb, c);
				formpanel.add(fb);
				if (form_str.indexOf(":" + ALL_FORMS[i] + ":") != -1) {
					forms[cnt] = fb;
					cnt++;
				} else {
					fb.setForeground(Colors.get("formbox.inactive"));
				}
			}
			activeform = 0;
			formpanel.validate();
			formpanel.repaint();
			if (applet.getParameter("pattern") != null)
				word_class = 4;
		}

	
	
	/**
	 * Encodes a string with the correct encoding for the language
	 * @see visl.rasmus.hitserver.hitclient.HitClient.getEncoding
	 */
	protected String encodeString(String str) {
		String encodedStr = null;
		try {
			// encodedStr = new String(str.getBytes(), hc.getEncoding()); //  Changed by Asad
			encodedStr = new String(str.getBytes(hc.getEncoding()), hc.getEncoding());
		} catch (Exception e) {
			System.err.println(hc.getEncoding() + " is not supported!");
			encodedStr = str;
		}

		return encodedStr;
	}
}
